"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const mocha_1 = require("@testdeck/mocha");
const chai_1 = require("chai");
const common_errors_1 = require("common-errors");
const src_1 = require("../src");
const KeyNotInObject_1 = require("./classes/KeyNotInObject");
const MissingChildType_1 = require("./classes/MissingChildType");
const Product_1 = require("./classes/Product");
const SkipIndex_1 = require("./classes/SkipIndex");
const Unknown_1 = require("./classes/Unknown");
/* eslint-disable @typescript-eslint/no-non-null-assertion */
let ClassToPlainArrayTest = 
// eslint-disable-next-line @typescript-eslint/no-unused-vars
class ClassToPlainArrayTest {
    async normal() {
        var _a;
        const expected = new Product_1.Product().setColor();
        const result = src_1.classToPlainArray(expected);
        // basic
        chai_1.expect(result.constructor).equals(Array);
        chai_1.expect(result).property('0', expected.id);
        // nested
        chai_1.expect(result).property('1').not.null;
        chai_1.expect((_a = result[1]) === null || _a === void 0 ? void 0 : _a.constructor).equals(Array);
        chai_1.expect(result).property('1').property('0', expected.color.name);
        // other values
        chai_1.expect(result).property('2', expected.price);
        chai_1.expect(result).property('3', expected.displayPrice);
    }
    async array() {
        var _a, _b;
        const array = [
            new Product_1.Product().setColor(),
            new Product_1.Product().setId(1).setColor().setPrice(1).setDisplayPrice('1'),
        ];
        const results = src_1.classToPlainArray(array);
        {
            const expected = array[0];
            const result = results[0];
            // basic
            chai_1.expect(result.constructor).equals(Array);
            chai_1.expect(result).property('0', expected.id);
            // nested
            chai_1.expect(result).property('1').not.null;
            chai_1.expect((_a = result[1]) === null || _a === void 0 ? void 0 : _a.constructor).equals(Array);
            chai_1.expect(result).property('1').property('0', expected.color.name);
            // other values
            chai_1.expect(result).property('2', expected.price);
            chai_1.expect(result).property('3', expected.displayPrice);
        }
        {
            const expected = array[1];
            const result = results[1];
            // basic
            chai_1.expect(result.constructor).equals(Array);
            chai_1.expect(result).property('0', expected.id);
            // nested
            chai_1.expect(result).property('1').not.null;
            chai_1.expect((_b = result[1]) === null || _b === void 0 ? void 0 : _b.constructor).equals(Array);
            chai_1.expect(result).property('1').property('0', expected.color.name);
            // other values
            chai_1.expect(result).property('2', expected.price);
            chai_1.expect(result).property('3', expected.displayPrice);
        }
    }
    emptyArray() {
        const result = src_1.classToPlainArray([]);
        chai_1.expect(result.constructor).equals(Array);
        chai_1.expect(result).length(0);
    }
    unknownClass() {
        chai_1.expect(() => src_1.classToPlainArray(new Unknown_1.UnknownClass())).throw(src_1.UnknownClassError);
    }
    keyNotInObject() {
        const obj = new KeyNotInObject_1.KeyNotInObject();
        delete obj.deleteThis;
        const result = src_1.classToPlainArray(obj);
        chai_1.expect(result.constructor).equals(Array);
        chai_1.expect(result).length(1);
        chai_1.expect(result).property('0', undefined);
    }
    missingChildType() {
        const result = src_1.classToPlainArray(new MissingChildType_1.ParentType());
        chai_1.expect(result.constructor).equals(Array);
        chai_1.expect(result).length(1);
        chai_1.expect(result).property('0', null);
    }
    undefinedArray() {
        const expected = new Product_1.Product().setColor();
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        expected.sizes = null;
        const result = src_1.classToPlainArray(expected);
        chai_1.expect(result).property('4').is.null;
    }
    arrayIsNotArray() {
        const expected = new Product_1.Product().setColor();
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        expected.sizes = 1;
        chai_1.expect(() => src_1.classToPlainArray(expected)).throw(common_errors_1.TypeError);
    }
    skipIndex() {
        const data = new SkipIndex_1.SkipIndex();
        data.child = new SkipIndex_1.SkipIndexChild();
        data.childArray = [new SkipIndex_1.SkipIndexChild(), new SkipIndex_1.SkipIndexChild()];
        const result = src_1.classToPlainArray(data);
        chai_1.expect(result.constructor).equals(Array);
        chai_1.expect(result).property('0', data.id);
        chai_1.expect(result).property('2', data.name);
        chai_1.expect(result).property('4').property('0', data.child.id);
        chai_1.expect(result).property('4').property('2', data.child.name);
        chai_1.expect(result).property('6').property('0').property('0', data.childArray[0].id);
        chai_1.expect(result).property('6').property('0').property('2', data.childArray[0].name);
        chai_1.expect(result).property('6').property('1').property('0', data.childArray[1].id);
        chai_1.expect(result).property('6').property('1').property('2', data.childArray[1].name);
    }
};
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ClassToPlainArrayTest.prototype, "normal", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ClassToPlainArrayTest.prototype, "array", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "emptyArray", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "unknownClass", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "keyNotInObject", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "missingChildType", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "undefinedArray", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "arrayIsNotArray", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "skipIndex", null);
ClassToPlainArrayTest = __decorate([
    mocha_1.suite()
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
], ClassToPlainArrayTest);
//# sourceMappingURL=classToPlainArray.test.js.map