"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const mocha_1 = require("@testdeck/mocha");
const chai_1 = require("chai");
const src_1 = require("../src");
const UnknownClassError_1 = require("./../src/exceptions/UnknownClassError");
const OutOFIndex_1 = require("./classes/OutOFIndex");
const Product_1 = require("./classes/Product");
const SkipIndex_1 = require("./classes/SkipIndex");
const Unknown_1 = require("./classes/Unknown");
let PlainArrayToClassTest = 
// eslint-disable-next-line @typescript-eslint/no-unused-vars
class PlainArrayToClassTest {
    normal() {
        var _a, _b;
        const expected = [1, ['blue'], 2.2, '2', [[1], [2]], [1, 2]];
        const result = src_1.plainArrayToClass(Product_1.Product, expected);
        // basic
        chai_1.expect(result.constructor).equals(Product_1.Product);
        chai_1.expect(result).property('id', expected[0]);
        // nested
        chai_1.expect(result).property('color').not.null;
        chai_1.expect((_a = result.color) === null || _a === void 0 ? void 0 : _a.constructor).equals(Product_1.Color);
        chai_1.expect(result).property('color').property('name', expected[1][0]);
        // test class-transform
        chai_1.expect(result).property('price', +(+expected[2]).toFixed());
        chai_1.expect(result).property('displayPrice', expected[3]);
        // test array
        chai_1.expect(result.sizes).not.null;
        chai_1.expect((_b = result.sizes) === null || _b === void 0 ? void 0 : _b.constructor).equal(Array);
        chai_1.expect(result.sizes).length(2);
        chai_1.expect(result.sizes).property('0').property('size', expected[4][0][0]);
        chai_1.expect(result.sizes).property('1').property('size', expected[4][1][0]);
        chai_1.expect(result.values).not.null;
        chai_1.expect(result.values).length(2);
        chai_1.expect(result.values).property('0').equal(expected[5][0]);
        chai_1.expect(result.values).property('1').equal(expected[5][1]);
    }
    empty() {
        const expected = new Product_1.Product();
        const result = src_1.plainArrayToClass(Product_1.Product, []);
        // basic
        chai_1.expect(result.constructor).equals(Product_1.Product);
        chai_1.expect(result).property('id', expected.id);
        // nested
        chai_1.expect(result).not.property('color');
        // test class-transform
        chai_1.expect(result).property('price', expected.price);
        chai_1.expect(result).property('displayPrice', expected.displayPrice);
    }
    nullValue() {
        var _a;
        const expected = [1, [null], 2.2];
        const result = src_1.plainArrayToClass(Product_1.Product, expected);
        // basic
        chai_1.expect(result.constructor).equals(Product_1.Product);
        chai_1.expect(result).property('id', expected[0]);
        // nested
        chai_1.expect(result).property('color').not.null;
        chai_1.expect((_a = result.color) === null || _a === void 0 ? void 0 : _a.constructor).equals(Product_1.Color);
        chai_1.expect(result).property('color').property('name', expected[1][0]);
        // test class-transform
        chai_1.expect(result).property('price', +(+expected[2]).toFixed());
        chai_1.expect(result).property('displayPrice', new Product_1.Product().displayPrice);
    }
    nullArray() {
        const expected = [1, null, 2.2];
        const result = src_1.plainArrayToClass(Product_1.Product, expected);
        // basic
        chai_1.expect(result.constructor).equals(Product_1.Product);
        chai_1.expect(result).property('id', expected[0]);
        // nested
        chai_1.expect(result).property('color').is.null;
        // test class-transform
        chai_1.expect(result).property('price', +expected[2].toFixed());
        chai_1.expect(result).property('displayPrice', new Product_1.Product().displayPrice);
    }
    emptyArray() {
        var _a;
        const expected = [1, [], 2.2];
        const result = src_1.plainArrayToClass(Product_1.Product, expected);
        // basic
        chai_1.expect(result.constructor).equals(Product_1.Product);
        chai_1.expect(result).property('id', expected[0]);
        // nested
        chai_1.expect(result).property('color').not.null;
        chai_1.expect((_a = result.color) === null || _a === void 0 ? void 0 : _a.constructor).equals(Product_1.Color);
        chai_1.expect(result).property('color').property('name', new Product_1.Color().name);
        // test class-transform
        chai_1.expect(result).property('price', +expected[2].toFixed());
        chai_1.expect(result).property('displayPrice', new Product_1.Product().displayPrice);
    }
    outOfIndex() {
        const expected = [1, 'out-of-index'];
        const result = src_1.plainArrayToClass(OutOFIndex_1.OutOfIndex, expected);
        // basic
        chai_1.expect(result.constructor).equals(OutOFIndex_1.OutOfIndex);
        chai_1.expect(result).property('id', expected[0]);
    }
    skipIndex() {
        var _a, _b;
        const expected = [1, null, 'n', null, [2, null, 'na'], null, [[3, null, 'na'], [4, null, 'na']]];
        const result = src_1.plainArrayToClass(SkipIndex_1.SkipIndex, expected);
        // basic
        chai_1.expect(result.constructor).equals(SkipIndex_1.SkipIndex);
        chai_1.expect(result).property('id', expected[0]);
        chai_1.expect(result.constructor).equals(SkipIndex_1.SkipIndex);
        chai_1.expect(result).property('name', expected[2]);
        // nested
        chai_1.expect(result).property('child').not.null;
        chai_1.expect((_a = result.child) === null || _a === void 0 ? void 0 : _a.constructor).equals(SkipIndex_1.SkipIndexChild);
        chai_1.expect(result).property('child').property('id', expected[4][0]);
        chai_1.expect(result).property('child').property('name', expected[4][2]);
        // nested array
        chai_1.expect(result).property('childArray').not.null;
        chai_1.expect((_b = result.childArray) === null || _b === void 0 ? void 0 : _b.constructor).equal(Array);
        chai_1.expect(result).property('childArray').property('0').property('constructor', SkipIndex_1.SkipIndexChild);
        chai_1.expect(result).property('childArray').property('0').property('id', expected[6][0][0]);
        chai_1.expect(result).property('childArray').property('0').property('name', expected[6][0][2]);
        chai_1.expect(result).property('childArray').property('1').property('constructor', SkipIndex_1.SkipIndexChild);
        chai_1.expect(result).property('childArray').property('1').property('id', expected[6][1][0]);
        chai_1.expect(result).property('childArray').property('1').property('name', expected[6][1][2]);
    }
    unknownClass() {
        chai_1.expect(() => src_1.plainArrayToClass(Unknown_1.UnknownClass, [])).throw(UnknownClassError_1.UnknownClassError);
    }
    unknownTypeWithoutValue() {
        chai_1.expect(() => src_1.plainArrayToClass(Unknown_1.UnknownTypeParent, [[]])).not.throw();
    }
};
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "normal", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "empty", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "nullValue", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "nullArray", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "emptyArray", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "outOfIndex", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "skipIndex", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "unknownClass", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], PlainArrayToClassTest.prototype, "unknownTypeWithoutValue", null);
PlainArrayToClassTest = __decorate([
    mocha_1.suite()
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
], PlainArrayToClassTest);
//# sourceMappingURL=plainArrayToClass.test.js.map